@@ -7,12 +7,6 @@ import sys
 import shutil
 import subprocess
 import tempfile
-try:
-    from io import StringIO
-except ImportError:
-    from StringIO import StringIO
-
-from unittest2 import skipIf, TestCase
 
 from pyflakes.messages import UnusedImport
 from pyflakes.reporter import Reporter
@@ -21,7 +15,12 @@ from pyflakes.api import (
     checkRecursive,
     iterSourceCode,
 )
-if sys.version_info >= (3,):
+from pyflakes.test.harness import TestCase
+
+if sys.version_info < (3,):
+    from cStringIO import StringIO
+else:
+    from io import StringIO
     unichr = chr
 
 
@@ -36,9 +35,18 @@ def withStderrTo(stderr, f, *args, **kwargs):
         sys.stderr = outer
 
 
+class Node(object):
+    """
+    Mock an AST node.
+    """
+    def __init__(self, lineno, col_offset=0):
+        self.lineno = lineno
+        self.col_offset = col_offset
+
+
 class LoggingReporter(object):
     """
-    Implementation of Reporter that just appends any errors to a list.
+    Implementation of Reporter that just appends any error to a list.
     """
 
     def __init__(self, log):
@@ -49,20 +57,16 @@ class LoggingReporter(object):
         """
         self.log = log
 
-
     def flake(self, message):
         self.log.append(('flake', str(message)))
 
-
     def unexpectedError(self, filename, message):
         self.log.append(('unexpectedError', filename, message))
 
-
     def syntaxError(self, filename, msg, lineno, offset, line):
         self.log.append(('syntaxError', filename, msg, lineno, offset, line))
 
 
-
 class TestIterSourceCode(TestCase):
     """
     Tests for L{iterSourceCode}.
@@ -81,14 +85,12 @@ class TestIterSourceCode(TestCase):
         fd.close()
         return fpath
 
-
     def test_emptyDirectory(self):
         """
         There are no Python files in an empty directory.
         """
         self.assertEqual(list(iterSourceCode([self.tempdir])), [])
 
-
     def test_singleFile(self):
         """
         If the directory contains one Python file, C{iterSourceCode} will find
@@ -97,7 +99,6 @@ class TestIterSourceCode(TestCase):
         childpath = self.makeEmptyFile('foo.py')
         self.assertEqual(list(iterSourceCode([self.tempdir])), [childpath])
 
-
     def test_onlyPythonSource(self):
         """
         Files that are not Python source files are not included.
@@ -105,7 +106,6 @@ class TestIterSourceCode(TestCase):
         self.makeEmptyFile('foo.pyc')
         self.assertEqual(list(iterSourceCode([self.tempdir])), [])
 
-
     def test_recurses(self):
         """
         If the Python files are hidden deep down in child directories, we will
@@ -120,7 +120,6 @@ class TestIterSourceCode(TestCase):
             sorted(iterSourceCode([self.tempdir])),
             sorted([apath, bpath, cpath]))
 
-
     def test_multipleDirectories(self):
         """
         L{iterSourceCode} can be given multiple directories.  It will recurse
@@ -136,7 +135,6 @@ class TestIterSourceCode(TestCase):
             sorted(iterSourceCode([foopath, barpath])),
             sorted([apath, bpath]))
 
-
     def test_explicitFiles(self):
         """
         If one of the paths given to L{iterSourceCode} is not a directory but
@@ -147,7 +145,6 @@ class TestIterSourceCode(TestCase):
                          [epath])
 
 
-
 class TestReporter(TestCase):
     """
     Tests for L{Reporter}.
@@ -162,14 +159,13 @@ class TestReporter(TestCase):
         """
         err = StringIO()
         reporter = Reporter(None, err)
-        reporter.syntaxError('foo.py', 'a problem', 3, 4, 'bad line of source')
-        self.assertEquals(
-            ("foo.py:3: a problem\n"
+        reporter.syntaxError('foo.py', 'a problem', 3, 7, 'bad line of source')
+        self.assertEqual(
+            ("foo.py:3:8: a problem\n"
              "bad line of source\n"
-             "     ^\n"),
+             "       ^\n"),
             err.getvalue())
 
-
     def test_syntaxErrorNoOffset(self):
         """
         C{syntaxError} doesn't include a caret pointing to the error if
@@ -179,12 +175,11 @@ class TestReporter(TestCase):
         reporter = Reporter(None, err)
         reporter.syntaxError('foo.py', 'a problem', 3, None,
                              'bad line of source')
-        self.assertEquals(
+        self.assertEqual(
             ("foo.py:3: a problem\n"
              "bad line of source\n"),
             err.getvalue())
 
-
     def test_multiLineSyntaxError(self):
         """
         If there's a multi-line syntax error, then we only report the last
@@ -195,17 +190,16 @@ class TestReporter(TestCase):
         lines = [
             'bad line of source',
             'more bad lines of source',
-            ]
+        ]
         reporter = Reporter(None, err)
-        reporter.syntaxError('foo.py', 'a problem', 3, len(lines[0]) + 5,
+        reporter.syntaxError('foo.py', 'a problem', 3, len(lines[0]) + 7,
                              '\n'.join(lines))
-        self.assertEquals(
-            ("foo.py:3: a problem\n" +
+        self.assertEqual(
+            ("foo.py:3:7: a problem\n" +
              lines[-1] + "\n" +
-             "     ^\n"),
+             "      ^\n"),
             err.getvalue())
 
-
     def test_unexpectedError(self):
         """
         C{unexpectedError} reports an error processing a source file.
@@ -213,8 +207,7 @@ class TestReporter(TestCase):
         err = StringIO()
         reporter = Reporter(None, err)
         reporter.unexpectedError('source.py', 'error message')
-        self.assertEquals('source.py: error message\n', err.getvalue())
-
+        self.assertEqual('source.py: error message\n', err.getvalue())
 
     def test_flake(self):
         """
@@ -223,10 +216,9 @@ class TestReporter(TestCase):
         """
         out = StringIO()
         reporter = Reporter(out, None)
-        message = UnusedImport('foo.py', 42, 'bar')
+        message = UnusedImport('foo.py', Node(42), 'bar')
         reporter.flake(message)
-        self.assertEquals(out.getvalue(), "%s\n" % (message,))
-
+        self.assertEqual(out.getvalue(), "%s\n" % (message,))
 
 
 class CheckTests(TestCase):
@@ -246,7 +238,6 @@ class CheckTests(TestCase):
         fd.close()
         return fpath
 
-
     def assertHasErrors(self, path, errorList):
         """
         Assert that C{path} causes errors.
@@ -256,10 +247,9 @@ class CheckTests(TestCase):
         """
         err = StringIO()
         count = withStderrTo(err, checkPath, path)
-        self.assertEquals(
+        self.assertEqual(
             (count, err.getvalue()), (len(errorList), ''.join(errorList)))
 
-
     def getErrors(self, path):
         """
         Get any warnings or errors reported by pyflakes for the file at C{path}.
@@ -274,12 +264,10 @@ class CheckTests(TestCase):
         count = checkPath(path, reporter)
         return count, log
 
-
     def test_legacyScript(self):
         from pyflakes.scripts import pyflakes as script_pyflakes
         self.assertIs(script_pyflakes.checkPath, checkPath)
 
-
     def test_missingTrailingNewline(self):
         """
         Source which doesn't end with a newline shouldn't cause any
@@ -289,18 +277,16 @@ class CheckTests(TestCase):
         fName = self.makeTempFile("def foo():\n\tpass\n\t")
         self.assertHasErrors(fName, [])
 
-
     def test_checkPathNonExisting(self):
         """
         L{checkPath} handles non-existing files.
         """
         count, errors = self.getErrors('extremo')
-        self.assertEquals(count, 1)
-        self.assertEquals(
+        self.assertEqual(count, 1)
+        self.assertEqual(
             errors,
             [('unexpectedError', 'extremo', 'No such file or directory')])
 
-
     def test_multilineSyntaxError(self):
         """
         Source which includes a syntax error which results in the raised
@@ -332,13 +318,12 @@ def baz():
 
         sourcePath = self.makeTempFile(source)
         self.assertHasErrors(
-            sourcePath, ["""\
-%s:8: invalid syntax
+            sourcePath,
+            ["""\
+%s:8:11: invalid syntax
     '''quux'''
-           ^
-"""
-                % (sourcePath,)])
-
+          ^
+""" % (sourcePath,)])
 
     def test_eofSyntaxError(self):
         """
@@ -349,11 +334,24 @@ def baz():
         self.assertHasErrors(
             sourcePath,
             ["""\
-%s:1: unexpected EOF while parsing
+%s:1:9: unexpected EOF while parsing
 def foo(
-         ^
+        ^
 """ % (sourcePath,)])
 
+    def test_eofSyntaxErrorWithTab(self):
+        """
+        The error reported for source files which end prematurely causing a
+        syntax error reflects the cause for the syntax error.
+        """
+        sourcePath = self.makeTempFile("if True:\n\tfoo =")
+        self.assertHasErrors(
+            sourcePath,
+            ["""\
+%s:2:7: invalid syntax
+\tfoo =
+\t     ^
+""" % (sourcePath,)])
 
     def test_nonDefaultFollowsDefaultSyntaxError(self):
         """
@@ -366,14 +364,14 @@ def foo(bar=baz, bax):
     pass
 """
         sourcePath = self.makeTempFile(source)
-        last_line = '        ^\n' if sys.version_info >= (3, 2) else ''
+        last_line = '       ^\n' if sys.version_info >= (3, 2) else ''
+        column = '8:' if sys.version_info >= (3, 2) else ''
         self.assertHasErrors(
             sourcePath,
             ["""\
-%s:1: non-default argument follows default argument
+%s:1:%s non-default argument follows default argument
 def foo(bar=baz, bax):
-%s""" % (sourcePath, last_line)])
-
+%s""" % (sourcePath, column, last_line)])
 
     def test_nonKeywordAfterKeywordSyntaxError(self):
         """
@@ -385,32 +383,36 @@ def foo(bar=baz, bax):
 foo(bar=baz, bax)
 """
         sourcePath = self.makeTempFile(source)
-        last_line = '             ^\n' if sys.version_info >= (3, 2) else ''
+        last_line = '            ^\n' if sys.version_info >= (3, 2) else ''
+        column = '13:' if sys.version_info >= (3, 2) else ''
         self.assertHasErrors(
             sourcePath,
             ["""\
-%s:1: non-keyword arg after keyword arg
+%s:1:%s non-keyword arg after keyword arg
 foo(bar=baz, bax)
-%s""" % (sourcePath, last_line)])
-
+%s""" % (sourcePath, column, last_line)])
 
     def test_invalidEscape(self):
         """
         The invalid escape syntax raises ValueError in Python 2
         """
+        ver = sys.version_info
         # ValueError: invalid \x escape
         sourcePath = self.makeTempFile(r"foo = '\xyz'")
-        if sys.version_info < (3,):
+        if ver < (3,):
             decoding_error = "%s: problem decoding source\n" % (sourcePath,)
         else:
+            last_line = '      ^\n' if ver >= (3, 2) else ''
+            # Column has been "fixed" since 3.2.4 and 3.3.1
+            col = 1 if ver >= (3, 3, 1) or ((3, 2, 4) <= ver < (3, 3)) else 2
             decoding_error = """\
-%s:1: (unicode error) 'unicodeescape' codec can't decode bytes in position 0-2: truncated \\xXX escape
+%s:1:7: (unicode error) 'unicodeescape' codec can't decode bytes \
+in position 0-%d: truncated \\xXX escape
 foo = '\\xyz'
-%s""" % (sourcePath, '       ^\n' if sys.version_info >= (3, 2) else '')
+%s""" % (sourcePath, col, last_line)
         self.assertHasErrors(
             sourcePath, [decoding_error])
 
-
     def test_permissionDenied(self):
         """
         If the source file is not readable, this is reported on standard
@@ -419,12 +421,11 @@ foo = '\\xyz'
         sourcePath = self.makeTempFile('')
         os.chmod(sourcePath, 0)
         count, errors = self.getErrors(sourcePath)
-        self.assertEquals(count, 1)
-        self.assertEquals(
+        self.assertEqual(count, 1)
+        self.assertEqual(
             errors,
             [('unexpectedError', sourcePath, "Permission denied")])
 
-
     def test_pyflakesWarning(self):
         """
         If the source file has a pyflakes warning, this is reported as a
@@ -432,12 +433,22 @@ foo = '\\xyz'
         """
         sourcePath = self.makeTempFile("import foo")
         count, errors = self.getErrors(sourcePath)
-        self.assertEquals(count, 1)
-        self.assertEquals(
-            errors, [('flake', str(UnusedImport(sourcePath, 1, 'foo')))])
+        self.assertEqual(count, 1)
+        self.assertEqual(
+            errors, [('flake', str(UnusedImport(sourcePath, Node(1), 'foo')))])
 
+    def test_encodedFileUTF8(self):
+        """
+        If source file declares the correct encoding, no error is reported.
+        """
+        SNOWMAN = unichr(0x2603)
+        source = ("""\
+# coding: utf-8
+x = "%s"
+""" % SNOWMAN).encode('utf-8')
+        sourcePath = self.makeTempFile(source)
+        self.assertHasErrors(sourcePath, [])
 
-    @skipIf(sys.version_info >= (3,), "not relevant")
     def test_misencodedFileUTF8(self):
         """
         If a source file contains bytes which cannot be decoded, this is
@@ -452,7 +463,6 @@ x = "%s"
         self.assertHasErrors(
             sourcePath, ["%s: problem decoding source\n" % (sourcePath,)])
 
-
     def test_misencodedFileUTF16(self):
         """
         If a source file contains bytes which cannot be decoded, this is
@@ -467,7 +477,6 @@ x = "%s"
         self.assertHasErrors(
             sourcePath, ["%s: problem decoding source\n" % (sourcePath,)])
 
-
     def test_checkRecursive(self):
         """
         L{checkRecursive} descends into each directory, finding Python files
@@ -489,9 +498,9 @@ x = "%s"
         self.assertEqual(warnings, 2)
         self.assertEqual(
             sorted(log),
-            sorted([('flake', str(UnusedImport(file1, 1, 'baz'))),
+            sorted([('flake', str(UnusedImport(file1, Node(1), 'baz'))),
                     ('flake',
-                     str(UnusedImport(file2, 1, 'contraband')))]))
+                     str(UnusedImport(file2, Node(1), 'contraband')))]))
 
 
 class IntegrationTests(TestCase):
@@ -514,7 +523,6 @@ class IntegrationTests(TestCase):
         package_dir = os.path.dirname(pyflakes.__file__)
         return os.path.join(package_dir, '..', 'bin', 'pyflakes')
 
-
     def runPyflakes(self, paths, stdin=None):
         """
         Launch a subprocess running C{pyflakes}.
@@ -542,7 +550,6 @@ class IntegrationTests(TestCase):
             stderr = stderr.decode('utf-8')
         return (stdout, stderr, rv)
 
-
     def test_goodFile(self):
         """
         When a Python source file is all good, the return code is zero and no
@@ -553,7 +560,6 @@ class IntegrationTests(TestCase):
         d = self.runPyflakes([self.tempfilepath])
         self.assertEqual(d, ('', '', 0))
 
-
     def test_fileWithFlakes(self):
         """
         When a Python source file has warnings, the return code is non-zero
@@ -563,8 +569,8 @@ class IntegrationTests(TestCase):
         fd.write("import contraband\n".encode('ascii'))
         fd.close()
         d = self.runPyflakes([self.tempfilepath])
-        self.assertEqual(d, ("%s\n" % UnusedImport(self.tempfilepath, 1, 'contraband'), '', 1))
-
+        expected = UnusedImport(self.tempfilepath, Node(1), 'contraband')
+        self.assertEqual(d, ("%s\n" % expected, '', 1))
 
     def test_errors(self):
         """
@@ -573,12 +579,13 @@ class IntegrationTests(TestCase):
         printed to stderr.
         """
         d = self.runPyflakes([self.tempfilepath])
-        self.assertEqual(d, ('', '%s: No such file or directory\n' % (self.tempfilepath,), 1))
-
+        error_msg = '%s: No such file or directory\n' % (self.tempfilepath,)
+        self.assertEqual(d, ('', error_msg, 1))
 
     def test_readFromStdin(self):
         """
         If no arguments are passed to C{pyflakes} then it reads from stdin.
         """
         d = self.runPyflakes([], stdin='import contraband'.encode('ascii'))
-        self.assertEqual(d, ("%s\n" % UnusedImport('<stdin>', 1, 'contraband'), '', 1))
+        expected = UnusedImport('<stdin>', Node(1), 'contraband')
+        self.assertEqual(d, ("%s\n" % expected, '', 1))
